function s = prim(strSurf, strDesc, strRflct, nPos, nAng, nAp, varargin),

% PRIM define a surface element object. 
% (part of the RAYTRACE toolbox.)
%
% PRIM(strSurf, strDesc, strRflct, nPos, nAng, nAp, ...). Create a surface 
% object of type strSurf, position nPos [x,y,z] orientation nAng = [rx, ry, rz] 
% reflect/refract flag (strRflct) and aperture radius nAp.  
% The variable strSurf can be 
%
%  strSurf /  additional parameters  
%
%   pupil       
%   plane      
%   image
%   ellipsoid  <a,b,c> 
%   bspline    err_tol,xml,desc
%   zernike    err_tol,dlist,desc
%   conic      <R,CC>,desc 
%   rkklein    refract,err_tol,<R,ep,R1,R2>,desc
%   rksicam    refract,err_tol,<R,ep,R1,R2>,desc
%
% PRIM('index', strDesc, nIndex). Create a refractive index object. 
%
% see also GET, SET, PRINT, PLAN
%
% Examples 
%
%   p = prim('index','air',1.335);
%
%   r0=7.5;a=0.0125;r1=1.5;r2=3.0;
%   p = prim('rkklein','klein-RK','refract',[0 0 0],[0 10 0],4,1e-9,r0,a,r1,r2); 
%
%   CV = 1/7; CC= 0; nAp = 5;
%   p = prim('conic', 'anterior cornea', 'reflect', [0 0 0], [0 0 0], nAp, CV, CC);

s = init_class;

% constructor
if  nargin==0 
    return;
end

s.type = strSurf;
s.description = strDesc;     

if (strcmp(lower(s.type), 'index')),
    s.args.name  = { 'index' }; 
    s.args.value = { strRflct };
    return
end

% standard options 
s.position      = nPos;
s.orientation   = nAng;        % orientation 
s.aperture      = nAp;
s.refraction    = strRflct;    % change to RefFlag = Reflect or Refract

% tbopt.lasterr = 0;
switch(lower(strSurf)),
  
  % analytic based primitives.     
  case { 'img','pupil' }
     % no additional parameters
      s.etol      = nan;
  case 'conic', 
      s.args.name = { 'CV', 'CC' };  
      s.etol      = nan;
  case 'sphere',     
      s.args.name = { 'R' };
      s.etol      = nan;
  case 'ellipsoid'
      s.args.name = { 'a','b','c' };
      s.etol      = nan;
    
  % surfaces based on a search approach. 
      
    % added 12/12/05 - modified 19/10/06
  case 'toric' 
      s.args.name = { 'Rx', 'Ry'  };
      s.etol      = varargin{1};
      varargin = varargin(2:end);
    % general surface models     
  case { 'zernike', 'bhwolf' }
      s.args.name  = { 'AP', 'COEFF'  };
      s.etol      = varargin{1};
      varargin = varargin(2:end);
  case { 'bspline' },
      s.args.name  = { 'COEFF' };
      s.etol      = varargin{1};
      varargin = varargin(2:end);
    % rkklein added 8/1/07
    % rksicam added 12/12/05 updated 11/1/07  
  case { 'rkklein', 'rksicam' }
      s.args.name  = { 'R', 'a', 'R1', 'R2' };      
      s.etol       = varargin{1}; 
      varargin = varargin(2:end);
  otherwise
        error('(create_element) requested to create unknown optic element.');
end

% do check on parameters
if (length(varargin) < length(s.args.name)),
    error('(PRIM) not enough parameters.'); 
elseif (length(varargin) > length(s.args.name)),
    error('(PRIM) too many parameters.'); 
end 

s.args.value =  varargin;
return

function s = init_class(),
    s.type          = [];
    s.description   = [];
    s.position      = [];
    s.orientation   = [];
    s.aperture      = [];
    s.refraction    = [];
    s.etol          = [];
    s.args.name     = [];
    s.args.value    = [];
    s = class(s, 'prim');
return