function [m_a, m_b] = getCylBnd(x, u, rp, varargin),

% GETCYLBND return a valid interval for locating intersections. 
%
% [a, b, i0] = GETCYLBND(FunFcn, nPos, nSlope, nAp, varargin),
%
% NOTE : 
%     a=nan, b=nan no intersection with bounding box (missed or coaxial)
%     a = 0  | b > 0   bounded by current location and b. 
%     a > 0  | b > 0   intersections possible     
%
% NB :
% 
%   there is a possibility for an error involving the  
%   fact that 10*eps has been subtracted from findrayintervals.  
%   This could lead to a missing the surface.  
%   
%   Possibly fix by adding a check that tests whether the interval    
%   calculated actually corresponds to finite values !!!


% determine the roots :
% ---------------------
%
%   u     = slopes 
%   x     = positions 
%   rp    = apertures 

m_a = []; m_b = [];

nd = (1:size(u,1))';

% parallel rays
a  = u(:,1).^2 + u(:,2).^2;
i = (a == 0); 
if any(i), 
   m_a(nd(i)) = nan;
   m_b(nd(i)) = nan;
   if (all(i)),
       return
   end
   % remaining
   i  = ~i;
   nd = nd(i);
   a  = a(i);
   x  = x(i,:);
   u  = u(i,:);
end  

% intervals  
b = 2*( x(:,1).*u(:,1) + x(:,2).*u(:,2) );
c = x(:,1).^2 + x(:,2).^2 - rp^2; 
d   = b.^2 - 4*a.*c; 
s   = -b./(2*a);

% ray misses cylinder completely
i = (d < 0);
if (any(i)),
    m_a(nd(i)) = nan;
    m_b(nd(i)) = nan;
    if (all(i)),
       return
    end 
    % reduce 
    i  = ~i;
    a  = a(i);
    d  = d(i);
    s  = s(i);
    nd = nd(i);
end 

% tangent ray
i = (d == 0)&(s >= 0);
if (any(i)),
    m_a(nd(i)) = 0;
    m_b(nd(i)) = s(i);
    if (all(i)),
       return
    end 
    % reduce 
    i  = ~i;
    a  = a(i);
    d  = d(i);
    s  = s(i);
    nd = nd(i);
end 

% two pairs (x1 > x2)
x1  = s + (sqrt(d))./(2*a);
x2  = s - (sqrt(d))./(2*a);

% (a > 0, b > 0)
i = (x1>0)&(x2>=0);
if  (any(i)),
    m_b(nd(i)) = x1(i);
    m_a(nd(i)) = x2(i);
    if (all(i)),
        return 
    end 
    % remove processed lines 
    i = ~i;
    x1 = x1(i);
    x2 = x2(i);
    nd = nd(i);
end 

% (a = 0, b > 0)
i = (x1>0)&(x2<0);
if  (any(i)),
    m_b(nd(i)) = x1(i);
    m_a(nd(i)) = 0;
    if (all(i)),
        return 
    end 
    % remove processed lines 
    i = ~i;
    x1 = x1(i);
    x2 = x2(i);
    nd = nd(i);
end 


% (a < 0, b < 0)
i = (x1 < 0);
if (any(i)),
   m_b(nd(i)) = nan;
   m_a(nd(i)) = nan;
   if (all(i)),
        return 
    end 
   % remove processed lines
   i = ~i;
   x1 = x1(i);
   x2 = x2(i);
   nd = nd(i);
end 

error('(GETCYLBND) Rays have not been processed.');
return