function y = csf_barten(u, w, b1, c, L),

% CSF_BARTEN Contrast Sensitivity Model introduced by
% Barten (1990).  
%
% CSF_BARTEN(u, w, b1, c, L).  Here u is the spatial frequency in 
% cycles/deg, w is the square root of the angular size of the 
% object in degrees, (b1, c) are fitting constants, and L is the 
% luminance in cd/m^2. This function reproduces the equation,
%
%   CSF(u) = a*u*exp(-b*u)*sqrt(1+c*exp(b*u))
%
% where 
%       a = (540*(1+0.7/L)^(-0.2))./(1+ 12./(w.*(1+u/3).^2));
%       b = b1*(1+100/L).^0.15
%
% Example : Campbell and Green (1965) [ Lambda = 632.8 nm ]
% using coefficients used by Lang et. al.
%
%    u = 0:1:60; 
%    L = 100; w = 0.7; b1 = 0.27; c = 0.06; 
%    y = csf_barten(u, w, b1, c, L);
%    loglog(u, y); title('CSF (Barten)');
%    xlabel('Frequency (cyc/deg)');
%    ylabel('Contrast Sensitivity');
%    grid on;
%    axis([1 60 1 1000 ]);

a = (540*(1+0.7/L)^(-0.2));
a = a./(1+ 12./(w*(1+u/3).^2));
b = b1*(1+100/L).^0.15;
y = a.*u.*exp(-b*u).*sqrt(1+c*exp(b*u));
return 
