function [ z0, varargout ] = focus(p0, s0, nApEntr, strMethod, M, varargin),

% FOCUS.  Estimate the position of minimum RMS OPD. 
%
% [z0, ... ] = FOCUS(p0, s0, nAp, strMethod, M, nZ). Here p is the lens 
% prescription object, s0 is  the axial image plane position, nGrid is the 
% resolution of the input beam, nAp is the entrance aperture and nZ is the 
% apex plane of the reference sphere. 
%
% strMethod can be etiher 'minRMSOPD' minimum RMS OPD over the wavefront,
% and 'minRMSR' minimum RMS geometric spot radius.
%
%
% Example,
%
%  p = eye_navarro; s0 = -6000; 
%  M = 64; nApEntr = 1; 
%  nExPup = getPupil(p, 6, 'exit');
% 
%  [z, e] = focus(p, s0, nApEntr, 'minRMSOPD', M, nExPup);
%  fprintf('Method 1. Minimum RMS-OPD\n'); 
%  fprintf('FOCAL PLANE = %2.6f\n', z); 
%  fprintf('RMS OPD     = %2.6f\n', e);
%
%  [z, e, ex2, ey2]   = focus(p, s0, nApEntr, 'minRMSR', M);
%  fprintf('Method 2. Minimum RMS Geometric Spot Radius\n'); 
%  fprintf('FOCAL PLANE = %2.6f\n', z); 
%  fprintf('GEO RMS X   = %2.6f\n', sqrt(ex2));
%  fprintf('GEO RMS Y   = %2.6f\n', sqrt(ey2));
%  fprintf('GEO RMS R   = %2.6f\n', e);
% 
%  nLambda = 0.5893e-3;
%  [z, e] = focus(p, s0, nApEntr, 'minVSOTF', M, 2*M, nExPup, nLambda);
%  fprintf('Method 3. Minimum VSOTF (Visual Strehl Ratio).\n');  
%  fprintf('FOCAL PLANE = %2.6f\n', z); 
%  fprintf('VSOTF       = %2.6f\n', e);
% 
%  nLambda = 0.5893e-3;
%  [z, e] = focus(p, s0, nApEntr, 'minVSMTF', M, 2*M, nExPup, nLambda);
%  fprintf('Method 3. Minimum VSMTF (Visual Strehl Ratio).\n');  
%  fprintf('FOCAL PLANE = %2.6f\n', z); 
%  fprintf('VSOTF       = %2.6f\n', e);
%
%  f = parax(p); f0 = pObj2Img(f.S, s0, 0) + f.BVP;
%  fprintf('Method 4. Paraxial Focal Plane\n'); 
%  fprintf('FOCAL PLANE = %2.6e\n', f0); 
%
%
%
% NOTE :
% 
% This function was tested against the monochromatic RMS OPD function 
% of OSLO-LT. The following differences were obtained.
%
% MIN RMS-OPD 
%
% ENTR. BEAM (MM) / OSLO (RMS OPD) / RAYTRACE / DIFF.
% -----------------------------------------------------
%   1        | 24.040605     | 24.0414   | -0.8
%   1.5      | 24.009048     | 24.0108   | -1.8
%   2        | 23.963178     | 23.9665   | -3.3
%   2.5      | 23.901258     | 23.9069   | -5.6
% 
%

    % chief ray (axial)
    nS  = [ 0 0 s0 ];
    
    % add a entrance beam filtering aperture.
    r0  = ray('GRID-FAN', nS, M, atan(-nApEntr/s0));
    p0  = seek(p0, 2); np1 = get(p0);     % filtering ap.
    np1 = set(np1, 'aperture', nApEntr); p0  = set(p0, np1);
    
    % Create input rays.
    [ r0, ndx ] = trace(r0, p0);
    
    switch(strMethod),
        
        case { 'minRMSOPD' },
    
            nZ = varargin{1};
            
            % chief ray
            rCh = ray(nS, [0 0 1]);
            rCh = trace(rCh, p0);    
            rCh = trans(rCh, nZ);             % ref. sph. plane             
            nCExOPL = get(rCh, 'OPL');        % default plane 
            nCExPos = get(rCh, 'position');   % default pos
            n1      = get(rCh, 'index');      % image space ref. index.                        
            
            [z0, e] = fminsearch(@err_RMSOPD, 20, [], rCh, r0, nCExPos, nCExOPL);
            varargout{1} = e;
            
        case  { 'minRMSR' },      
            
            z0 = fminsearch(@err_RMSR, 20, [], r0);
            [e, ex2, ey2] = err_RMSR(z0, r0);
            
            varargout{1} = e;
            varargout{2} = ex2;
            varargout{3} = ey2;
            
        case   { 'minVSOTF' },

            N       = varargin{1};
            nZ      = varargin{2};
            nLambda = varargin{3};
                                    
            % chief ray
            rCh = ray(nS, [0 0 1]);
            rCh = trace(rCh, p0);    
            rCh = trans(rCh, nZ);   % ref. sph. plane             

            z0 = fminsearch(@err_VSOTF, 20, [], rCh, r0, nLambda, M, N, ndx);
            e  = err_VSOTF(z0, rCh, r0, nLambda, M, N, ndx);
            varargout{1} = e;
            
        case   { 'minVSMTF' },
            
            N       = varargin{1};
            nZ      = varargin{2};
            nLambda = varargin{3};
                        
            % chief ray
            rCh = ray(nS, [0 0 1]);
            rCh = trace(rCh, p0);    
            rCh = trans(rCh, nZ);   % ref. sph. plane             

            z0 = fminsearch(@err_VSMTF, 20, [], rCh, r0, nLambda, M, N, ndx);
            e  = err_VSMTF(z0, rCh, r0, nLambda, M, N, ndx);
            varargout{1} = e;
            
            
        otherwise,
            error('(FOCUS) Unknown minimization approach requested ');
    end
            
return

function e = err_RMSOPD(z0, rCh, rGrid, nCExPos, nCExOPL),

    n1        = get(rCh, 'index');      % image space ref. index.  
    rCh       = trans(rCh, z0);             
    nCImgOPL  = get(rCh, 'OPL');
    nCImgPos  = get(rCh, 'position');
    nR        = norm(nCExPos - nCImgPos);  % radius   
    
    % Create a grid of matrix wavefront errors.
    p = prim('sphere', 'ref. sph.', 'refract', nCExPos, [ 0 0 0 ], nR, nR);
    rGrid = raynsect(rGrid, p, n1, 'bck');   
    nOPL = nCExOPL-get(rGrid, 'OPL');
    e    = std(nOPL); % RMS-OPD
return


function [e, ex2, ey2] = err_RMSR(z0, r),
    r   = trans(r, z0);             
    z   = get(r,'position');
    ex2 = var(z(:,1));
    ey2 = var(z(:,2));
    e   = sqrt(ex2 + ey2);
return


function e = err_VSOTF(z0, rCh, rGrid, nLambda, M, N, ndx),

    % assumes the chief ray is at the ref. plane 
    n1          = get(rCh, 'index');
    nCExPos     = get(rCh, 'position');
    nCExOPL     = get(rCh, 'OPL');
    rCh         = trans(rCh, z0);
    nCImgOPL    = get(rCh,'OPL');
    nCImgPos    = get(rCh,'position');
    nR          = norm(nCExPos - nCImgPos);
    
    % wavefront & pupil mask
    [nTX, nTY ] = Vec2EulYX(nCImgPos-nCExPos);
    p  = prim('sphere', 'ref. sph.', 'refract', nCExPos, ...
                [ nTX nTY 0 ], nR, nR);  % ref. sph.
    rGrid = raynsect(rGrid, p, n1, 'bck'); 
    w = zeros(M); pupil = w; pupil(ndx) = 1;
    w(ndx) = nCExOPL - get(rGrid, 'OPL');
    
    % calculated diameter
    nX   = get(rGrid, 'position');
    nDX1 = max(nX(:,1)); nDX2 = min(nX(:,1));
    nDY1 = max(nX(:,2)); nDY2 = min(nX(:,2));   
    nD = mean([ abs(nDX2-nDX1) abs(nDY2-nDY1) ]);
    
    % frequency variables (scaling)
    x   = getApScaling(nR, nD, nLambda, M, N);
    fx  = (x.FC2)*linspace(0, 1, M);
    fx  = repmat(fx, M, 1);
        
    % frequency in cyc/deg
    theta = atan(1/nR)*(180/pi);
    u     = sqrt(fx.^2 + (fx').^2); 
    u     = u/theta;
    
    % retinal csf (fx, fy) - symmetric
    L = 100; w0 = 0.7; b1 = 0.27; c = 0.06; 
    CSF = csf_barten(u, w0, b1, c, L);
    % CSF = CSF/max(abs(CSF(:)));    
    
    % calculate VSOTF
    p       = pupil.*exp(2*pi*j*w);
    nOTF    = real(acorr2(p)); 
    nOTF    = nOTF(1:M, 1:M); nOTF  = rot90(rot90(nOTF)); % /max(abs(nOTF(:)));
    nDOTF   = acorr2(pupil);                
    nDOTF   = nDOTF(1:M, 1:M); nDOTF  = rot90(rot90(nDOTF)); % /max(abs(nDOTF(:)));
    e       = sum(CSF(:).*nOTF(:))/sum(CSF(:).*nDOTF(:));
    e       = real(e);
    e       = -e;
    
    % fprintf('focal plane = %2.6f\n', z0);
    % fprintf('error       = %2.6f\n', e);        
return


function e = err_VSMTF(z0, rCh, rGrid, nLambda, M, N, ndx),

    % assumes the chief ray is at the ref. plane 
    n1          = get(rCh, 'index');
    nCExPos     = get(rCh, 'position');
    nCExOPL     = get(rCh, 'OPL');
    rCh         = trans(rCh, z0);
    nCImgOPL    = get(rCh,'OPL');
    nCImgPos    = get(rCh,'position');
    nR          = norm(nCExPos - nCImgPos);
    
    % wavefront & pupil mask
    [nTX, nTY ] = Vec2EulYX(nCImgPos-nCExPos);
    p  = prim('sphere', 'ref. sph.', 'refract', nCExPos, ...
                [ nTX nTY 0 ], nR, nR);  % ref. sph.
    rGrid = raynsect(rGrid, p, n1, 'bck'); 
    w = zeros(M); pupil = w; pupil(ndx) = 1;
    w(ndx) = nCExOPL - get(rGrid, 'OPL');   
    
    % calculated diameter
    nX   = get(rGrid, 'position');
    nDX1 = max(nX(:,1)); nDX2 = min(nX(:,1));
    nDY1 = max(nX(:,2)); nDY2 = min(nX(:,2));   
    nD = mean([ abs(nDX2-nDX1) abs(nDY2-nDY1) ]);
    
    % frequency variables (scaling)
    x   = getApScaling(nR, nD, nLambda, M, N);
    fx  = (x.FC2)*linspace(0, 1, M);
    fx  = repmat(fx, M, 1);
        
    % frequency in cyc/deg
    theta = atan(1/nR)*(180/pi);
    u     = sqrt(fx.^2 + (fx').^2); 
    u     = u/theta;
    
    % retinal csf (fx, fy) - symmetric
    L = 100; w0 = 0.7; b1 = 0.27; c = 0.06; 
    CSF = csf_barten(u, w0, b1, c, L);
    % CSF = CSF/max(abs(CSF(:)));             % scaling shouldn't matter
        
    % calculate VSMTF
    p       = pupil.*exp(2*pi*j*w);
    nMTF    = abs(acorr2(p)); % nMTF = nMTF/max(nMTF(:));      
    nMTF    = nMTF(1:M, 1:M); nMTF  = rot90(rot90(nMTF));

    nDMTF   = abs(acorr2(pupil)); % nDMTF = nDMTF/max(nDMTF(:));
    nDMTF   = nDMTF(1:M, 1:M); nDMTF  = rot90(rot90(nDMTF));
    
    e       = sum(CSF(:).*nMTF(:))/sum(CSF(:).*nDMTF(:));
    e       = -e;
    
    
    % fprintf('focal plane = %2.6e\n', z0);
    % fprintf('error       = %2.6e\n', e);
    
    
return


function x = getApScaling(nR, nD, nLambda, M, N);
    
    % get freq. variables 
    x.DY1 = nD/M;                    % exit pupil cell size
    x.FS1 = 1/x.DY1;                 % Sampling rate 
    x.FC1 = 1/(2*x.DY1);             % Cutoff freq. (exit pupil) 
    x.DY2 = (nLambda*nR)/(N*x.DY1);  % image plane cell size  
    x.FC2 = 1/(2*x.DY2);                 % Cutoff freq. (image plane)
    
return