function [w, pupil, nD] = pfn(p0, s0, nGrid, nApEntr, nZ, nLambda0),

% PFN Produce a pupil function from the lens prescription
% 
% [w, pupil, nD] = PFN(p, s0, nGrid, nApEntr, nX).  Here p is the lens prescription, 
% s0 is the axial source point (should be -ve), nGrid is the dimension of the aperture.  
% nApEntr is the aperture of the entrance beam, and nZ is the axial position of 
% the apex of the reference sphere.   The wavefront error is returned in
% wave units. 
%
% The process is as follows : 
%
%       1 - trace a chief ray to the image plane 
%       2 - send it back to the exit pupil plane 
%       3 - form the ref. sphere about this point
%       4 - trace a grid of rays (equally spaced 
%           in the entrance beam).
%       5 - trace these rays back to the exit pupil.
%       6 - subtract chief ray OPL.
%
% NOTE : 
%        1/ test eye should end with an IMG surface 
%
% Example
% 
%   nApEntr    = 2;                            % entr. beam 
%   nApID      = 6;                            % ap. stop ID
%   s0         = -6000; 
%   nGrid      = 128;                          % ray-traced grid
%   nLambda    = 0.5893e-3; % mm
%   p          = eye_navarro;                       
%   nApZ       = getPupil(p, nApID, 'exit');   % axial pos. of ref. sph.
%   [w, pupil] = pfn(p, s0, nGrid, nApEntr, nApZ);  % OPD units

    % reference sphere details 
    nS = [ 0 0 s0 ];
    [nR, nCExPos, nCPos, nCExOPL, nCOPL ] = getRefSph(p0, s0, nZ); 

    fprintf('OPL on Img. Pln. = %2.6f\n', nCOPL);
    fprintf('OPL on Ref. Sph  = %2.6f\n', nCExOPL);
    fprintf('Exit Pup. Cntr.  = (%2.6f, %2.6f, %2.6f)\n', nCExPos);
    fprintf('Ref. Sph. (R)    = %2.6f\n', nR);
             
    % aim for nApEntr on surface nID = 2 (1st surface)
    
    d = atan(-nApEntr/s0); % OSLO method for Entrance Beam  
    % d = getEntrBeam(p0, [0 0 s0], 2, nApEntr, 1e-6);  % unit vectors 
    % d = atan(d(1,2)/d(1,3));
    r0  = ray('GRID-FAN', nS, nGrid, d);
    
    % set an aperture
    p0 = seek(p0, 2); np1 = get(p0); 
    np1 = set(np1, 'aperture', nApEntr);
    p0 = set(p0, np1);
        
    
    % Create a grid of matrix wavefront errors.
    [r, j, x] = trace(r0, p0); 
    [nTX, nTY] = Vec2EulYX(vunit(nCPos-nCExPos));  % ref. ray to Eul(YX) angles
    p = prim('sphere', 'ref. sph.', 'refract', nCExPos, [ nTX nTY 0 ], nR, nR);
    n1 = x.n1; r = raynsect(r, p, n1, 'bck');
    nOPL = get(r, 'OPL');
    nX   = get(r, 'position');
        
    % calculated diameter
    nDX1 = max(nX(:,1)); nDX2 = min(nX(:,1));
    nDY1 = max(nX(:,2)); nDY2 = min(nX(:,2));   
    nD = mean([ abs(nDX2-nDX1) abs(nDY2-nDY1) ]);
        
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % grid
    
    w     = zeros(nGrid); pupil = w;
    w(j)  = -(nOPL -nCExOPL)/nLambda0;
    pupil(j) = 1;
    
    % nMaxOPL = max(w(:)); 
    % nMinOPL = min(w(:));
    % nLambda = 589.3e-6; % mm
    
    % fprintf('PFN OUTPUT.\n');
    % fprintf('--------------------------------\n');    
    
    % fprintf('Max. OPL   = %2.6e\n', nMaxOPL);
    % fprintf('Min. OPL   = %2.6e\n', nMinOPL);
    % fprintf('P-V error  = %2.6e\n', nMaxOPL-nMinOPL);      
    
    % fprintf('Lambda     = %2.6e\n', nLambda);
    % fprintf('Max. Waves = %2.6f\n', nMaxOPL/nLambda);
    % fprintf('Min. Waves = %2.6f\n', nMinOPL/nLambda);
    % fprintf('P-V (Waves)= %2.6f\n', (nMaxOPL-nMinOPL)/nLambda);      
    
    % nRMSOPD = std(nOPL/nLambda);
    % fprintf('RMS OPD    = %2.6f\n', nRMSOPD);
    
return