function [z, x, y] = pfn2psf(w, pupil, nD, nR, nLambda, nIndex)

% PFN2PSF Normalized Point Spread Function (PSF) from the pupil function.  
%
% PFN2PSF(w, pupil).  Here w is the wavefront error across the pupil (given
% in wave units), whilst the variable pupil is the mask. 
%
% PFN2PSF(w, pupil, nD, nR, nLambda, nIndex). Additional information used 
% to compute the scaling factors.  
%
% Extra information required,
%
%   nLambda   Wavelength (Image Space)          
%   nIndex    Image space index     
%   nD        Exit pupil diameter        
%   nR        Radius of reference sphere
%  
% Example,
%
%   [w, pupil] = pfnAiry(256, 1);
%   nD = 3; nR = 20; nLambda = 0.5893e-3; nIndex = 1.336;
%   [z, x, y] = pfn2psf(w, pupil, nD, nR, nLambda, nIndex);
%
% NOTES :
%
% This code originally appeared in code posted by Prof. John Loomis at :
% http://www.engr.udayton.edu/faculty/jloomis/eop601/matlab/psf/fft_psf.html
% last accessed : Feb 2007
%

M       = size(w, 1);           % PFN GRID (rays traced)
N       = 2*M;                  % FFT grid size.

z = zeros(N,N);
z(1:M, 1:M) = pupil.*exp(2*pi*j*w);

z = fftshift(z);
z = fft2(z);
z = fftshift(z);
z = z.*conj(z);
zmax = max(max(z));
z = z/zmax;

if (nargin == 2),
    % no parameters passed
    x = []; y = [];
    return
end

% determine image space factors
dY1     = nD/M;                  % exit pupil cell size
dY2     = (nLambda*nR)/(N*dY1);  % image plane cell size  

% determine position scaling
x  = (1:N)*dY2; x = repmat(x, N, 1);
y = x';

return